Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type Builder ¶
type Builder struct {
Compile
CaddyVersion string `json:"caddy_version,omitempty"`
Plugins []Dependency `json:"plugins,omitempty"`
Replacements []Replace `json:"replacements,omitempty"`
TimeoutGet time.Duration `json:"timeout_get,omitempty"`
TimeoutBuild time.Duration `json:"timeout_build,omitempty"`
RaceDetector bool `json:"race_detector,omitempty"`
SkipCleanup bool `json:"skip_cleanup,omitempty"`
}
Builder can produce a custom Caddy build with the configuration it represents.
type Compile ¶ added in v0.1.2
Compile contains parameters for compilation.
func SupportedPlatforms ¶ added in v0.1.2
SupportedPlatforms runs `go tool dist list` to make a list of possible build targets.
func (Compile) CgoEnabled ¶ added in v0.1.4
CgoEnabled returns "1" if c.Cgo is true, "0" otherwise. This is used for setting the CGO_ENABLED env variable.
type Dependency ¶
type Dependency struct {
// The name (path) of the Go module. If at a version > 1, it
// should contain semantic import version suffix (i.e. "/v2").
// Used with `go get`
ModulePath string `json:"module_path,omitempty"`
// The version of the Go module, like used with `go get`.
Version string `json:"version,omitempty"`
}
Dependency pairs a Go module path with a version.
Click to show internal directories.
Click to hide internal directories.